#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->lineEdit->setValidator(new QDoubleValidator(0, 0, 2, this));



    mvSettings.sync();
    initialSettings();
    if (mvSettings.value("AutoUpdate").toBool()) {
        updateCurrency();
    }
    updateButtons();
    mvTotal = 0;
    updateValue(0);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::initialSettings()
{
    if (!mvSettings.contains("AutoUpdate")) {
        mvSettings.setValue("AutoUpdate", true);
    }

    if (mvSettings.value("Restore").toInt() < 1) {
        mvSettings.setValue("Restore", 1);
        mvSettings.setValue("Currency1", "EUR");
        mvSettings.setValue("Currency2", "USD");
        mvSettings.setValue("TaxRate", 20);
        mvSettings.setValue("TipRate", 10);
        mvSettings.setValue("Currency/EUR", 1);
        mvSettings.setValue("Currency/USD", 1.4374);
        mvSettings.setValue("Currency/JPY", 130.76);
        mvSettings.setValue("Currency/BGN", 1.9558);
        mvSettings.setValue("Currency/CZK", 25.947);
        mvSettings.setValue("Currency/DKK", 7.4420);
        mvSettings.setValue("Currency/EEK", 15.6466);
        mvSettings.setValue("Currency/GBP", 0.88105);
        mvSettings.setValue("Currency/HUF", 266.80);
        mvSettings.setValue("Currency/LTL", 3.4528);
        mvSettings.setValue("Currency/LVL", 0.7085);
        mvSettings.setValue("Currency/PLN", 4.0399);
        mvSettings.setValue("Currency/RON", 4.1043);
        mvSettings.setValue("Currency/SEK", 10.1430);
        mvSettings.setValue("Currency/CHF", 1.4756);
        mvSettings.setValue("Currency/NOK", 8.1430);
        mvSettings.setValue("Currency/HRK", 7.2890);
        mvSettings.setValue("Currency/RUB", 42.5015);
        mvSettings.setValue("Currency/TRY", 2.0900);
        mvSettings.setValue("Currency/AUD", 1.5547);
        mvSettings.setValue("Currency/BRL", 2.5438);
        mvSettings.setValue("Currency/CAD", 1.4752);
        mvSettings.setValue("Currency/CNY", 9.8130);
        mvSettings.setValue("Currency/HKD", 11.1536);
        mvSettings.setValue("Currency/IDR", 13245.61);
        mvSettings.setValue("Currency/INR", 65.7000);
        mvSettings.setValue("Currency/KRW", 1614.29);
        mvSettings.setValue("Currency/MXN", 18.2790);
        mvSettings.setValue("Currency/MYR", 4.8002);
        mvSettings.setValue("Currency/NZD", 1.9497);
        mvSettings.setValue("Currency/PHP", 65.867);
        mvSettings.setValue("Currency/SGD", 1.9984);
        mvSettings.setValue("Currency/THB", 47.244);
        mvSettings.setValue("Currency/ZAR", 10.6588);
        mvSettings.setValue("CurrencyName/EUR", "Euro");
        mvSettings.setValue("CurrencyName/USD", "US dollar");
        mvSettings.setValue("CurrencyName/JPY", "Japanese yen");
        mvSettings.setValue("CurrencyName/BGN", "Bulgarian lev");
        mvSettings.setValue("CurrencyName/CZK", "Czech koruna");
        mvSettings.setValue("CurrencyName/DKK", "Danish krone");
        mvSettings.setValue("CurrencyName/EEK", "Estonian kroon");
        mvSettings.setValue("CurrencyName/GBP", "Pound sterling");
        mvSettings.setValue("CurrencyName/HUF", "Hungarian forint");
        mvSettings.setValue("CurrencyName/LTL", "Lithuanian litas");
        mvSettings.setValue("CurrencyName/LVL", "Latvian lats");
        mvSettings.setValue("CurrencyName/PLN", "Polish zloty");
        mvSettings.setValue("CurrencyName/RON", "Romanian leu");
        mvSettings.setValue("CurrencyName/SEK", "Swedish krona");
        mvSettings.setValue("CurrencyName/CHF", "Swiss franc");
        mvSettings.setValue("CurrencyName/NOK", "Norwegian krone");
        mvSettings.setValue("CurrencyName/HRK", "Croatian kuna");
        mvSettings.setValue("CurrencyName/RUB", "Russian rouble");
        mvSettings.setValue("CurrencyName/TRY", "Turkish lira");
        mvSettings.setValue("CurrencyName/AUD", "Australian dollar");
        mvSettings.setValue("CurrencyName/BRL", "Brasilian real");
        mvSettings.setValue("CurrencyName/CAD", "Canadian dollar");
        mvSettings.setValue("CurrencyName/CNY", "Chinese yuan renminbi");
        mvSettings.setValue("CurrencyName/HKD", "Hong Kong dollar");
        mvSettings.setValue("CurrencyName/IDR", "Indonesian rupiah");
        mvSettings.setValue("CurrencyName/INR", "Indian rupee");
        mvSettings.setValue("CurrencyName/KRW", "South Korean won");
        mvSettings.setValue("CurrencyName/MXN", "Mexican peso");
        mvSettings.setValue("CurrencyName/MYR", "Malaysian ringgit");
        mvSettings.setValue("CurrencyName/NZD", "New Zealand dollar");
        mvSettings.setValue("CurrencyName/PHP", "Philippine peso");
        mvSettings.setValue("CurrencyName/SGD", "Singapore dollar");
        mvSettings.setValue("CurrencyName/THB", "Thai baht");
        mvSettings.setValue("CurrencyName/ZAR", "South African rand");
    }

}

void MainWindow::updateValue(int actionType)
{
    double currentValue = ui->lineEdit->text().toDouble();

    switch (mvAction) {
    case 0:
        mvTotal = currentValue;
        break;
    case 1:
        mvTotal += currentValue;
        break;
    case 2:
        mvTotal -= currentValue;
        break;
    case 3:
        mvTotal *= currentValue;
        break;
    case 4:
        mvTotal /= currentValue;
        break;
    }

    QString labelText = ui->label->text();
    QString formatRate;
    float addRate;


    switch (actionType) {
    case 5:
        addRate = mvTotal / 100 * mvSettings.value("TipRate").toDouble();
        mvTotal += addRate;
        mvAction = 0;
        formatRate.setNum(addRate, 'f', 2);
        labelText = "Tip " + formatRate;
        ui->labelSmall->setText(labelText);
        break;
    case 6:
        addRate = mvTotal / 100 * mvSettings.value("TaxRate").toDouble();
        mvTotal += addRate;
        mvAction = 0;
        formatRate.setNum(addRate, 'f', 2);
        labelText = "Tax " + formatRate;
        ui->labelSmall->setText(labelText);
        break;
    default:
        mvAction = actionType;
        break;
    }

    mvClear = true;

    ui->lineEdit->setText(QString::number(mvTotal, 'f', 2));
}

void MainWindow::updateScreen(int valueType)
{
    if (mvClear) {
        ui->lineEdit->clear();
    }

    if (mvUpdated) {
        ui->label->clear();
        mvUpdated = false;
    }

    ui->labelSmall->clear();

    switch (valueType) {
    case 0:
        ui->lineEdit->insert("0");
        break;
    case 1:
        ui->lineEdit->insert("1");
        break;
    case 2:
        ui->lineEdit->insert("2");
        break;
    case 3:
        ui->lineEdit->insert("3");
        break;
    case 4:
        ui->lineEdit->insert("4");
        break;
    case 5:
        ui->lineEdit->insert("5");
        break;
    case 6:
        ui->lineEdit->insert("6");
        break;
    case 7:
        ui->lineEdit->insert("7");
        break;
    case 8:
        ui->lineEdit->insert("8");
        break;
    case 9:
        ui->lineEdit->insert("9");
        break;
    case 10:
        ui->lineEdit->insert(".");
        break;
    case 11:
        ui->lineEdit->backspace();
        break;
    case 12:
        ui->lineEdit->clear();
        ui->label->clear();
        ui->labelSmall->clear();
        mvTotal = 0;
        break;
    }

    mvClear = false;
}

void MainWindow::convertCurrency(QString fromCurrency, QString toCurrency)
{
    double currentValue = ui->lineEdit->text().toDouble();
    double fromRate;
    double toRate;
    double conversionRate;
    bool fromEuro;
    bool toEuro;

    if (fromCurrency == "EUR") {
        fromEuro = true;
    } else {
        fromRate = mvSettings.value("Currency/" + fromCurrency).toDouble();
        fromEuro = false;
    }

    if (toCurrency == "EUR") {
        toEuro = true;
    } else {
        toRate = mvSettings.value("Currency/" + toCurrency).toDouble();
        toEuro = false;
    }

    if (fromEuro) {
        currentValue *= toRate;
    } else if (toEuro) {
        conversionRate = 1 / fromRate;
        currentValue *= conversionRate;
    } else {
        conversionRate = 1 / fromRate;
        conversionRate *= toRate;
        currentValue *= conversionRate;
    }

    mvTotal = currentValue;
    ui->labelSmall->clear();
    ui->lineEdit->setText(QString::number(mvTotal, 'f', 2));
    mvClear = true;
}

void MainWindow::updateCurrency()
{
    QNetworkAccessManager manager;
    QEventLoop loop;
    QTimer *timer = new QTimer(this);
    QNetworkRequest *request = new QNetworkRequest(QUrl("http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"));
    QNetworkReply *reply = manager.get(*request);

    connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    connect(timer, SIGNAL(timeout()), &loop, SLOT(quit()));

    ui->label->setText("Updating...");

    loop.exec();
    timer->start(10000);
    xml.clear();
    xml.addData(reply->readAll());
 
    delete reply;

    while (!xml.atEnd()) {
        if (xml.isStartElement()) {
            if (xml.name() == "Cube") {
                QXmlStreamAttributes attrs = xml.attributes();
                QStringRef currencyTime = attrs.value("time");
                QStringRef currencyName = attrs.value("currency");
                QStringRef currencyRate = attrs.value("rate");
                if (!currencyTime.isEmpty()) {
                    mvSettings.setValue("CurrencyTime", currencyTime.toString());
                }
                if (!currencyName.isEmpty()) {
                    mvSettings.setValue("Currency/" + currencyName.toString(), currencyRate.toString());
                }
            }
        }
        xml.readNext();
    }
    
    mvSettings.setValue("Currency/EUR", 1);

    QDate date = mvSettings.value("CurrencyTime").toDate();
    QString updated = date.toString();

    ui->label->setText(updated);
    mvUpdated = true;
}

void MainWindow::updateButtons()
{
    QString currency1 = mvSettings.value("Currency1").toString();
    QString currency2 = mvSettings.value("Currency2").toString();

    currency1 += " -> " + mvSettings.value("Currency2").toString();
    currency2 += " -> " + mvSettings.value("Currency1").toString();

    ui->pushButton_Currency1->setText(currency1);
    ui->pushButton_Currency2->setText(currency2);
}

void MainWindow::on_pushButton_1_clicked()
{
    updateScreen(1);
}

void MainWindow::on_pushButton_2_clicked()
{
    updateScreen(2);
}

void MainWindow::on_pushButton_3_clicked()
{
    updateScreen(3);
}

void MainWindow::on_pushButton_4_clicked()
{
    updateScreen(4);
}

void MainWindow::on_pushButton_5_clicked()
{
    updateScreen(5);
}

void MainWindow::on_pushButton_6_clicked()
{
    updateScreen(6);
}

void MainWindow::on_pushButton_7_clicked()
{
    updateScreen(7);
}

void MainWindow::on_pushButton_8_clicked()
{
    updateScreen(8);
}

void MainWindow::on_pushButton_9_clicked()
{
    updateScreen(9);
}

void MainWindow::on_pushButton_0_clicked()
{
    updateScreen(0);
}

void MainWindow::on_pushButton_Dot_clicked()
{
    updateScreen(10);
}

void MainWindow::on_pushButton_Del_clicked()
{
    updateScreen(11);
}

void MainWindow::on_pushButton_Clear_clicked()
{
    updateScreen(12);
}

void MainWindow::on_pushButton_Enter_clicked()
{
    updateValue(0);
}

void MainWindow::on_pushButton_Plus_clicked()
{
    updateValue(1);
}

void MainWindow::on_pushButton_Minus_clicked()
{
    updateValue(2);
}

void MainWindow::on_pushButton_Multiply_clicked()
{
    updateValue(3);
}

void MainWindow::on_pushButton_Divide_clicked()
{
    updateValue(4);
}

void MainWindow::on_pushButton_Tip_clicked()
{
    updateValue(5);
}

void MainWindow::on_pushButton_Tax_clicked()
{
    updateValue(6);
}


void MainWindow::on_pushButton_Currency1_clicked()
{
    QString currency1 = mvSettings.value("Currency1").toString();
    QString currency2 = mvSettings.value("Currency2").toString();
    convertCurrency(currency1, currency2);
    ui->label->setText(currency2);
}

void MainWindow::on_pushButton_Currency2_clicked()
{
    QString currency1 = mvSettings.value("Currency1").toString();
    QString currency2 = mvSettings.value("Currency2").toString();
    convertCurrency(currency2, currency1);
    ui->label->setText(currency1);
}

void MainWindow::on_pushButton_SetCurrency_clicked()
{
    currencyDialog = new Currency;
    currencyDialog->exec();
    updateButtons();
}

void MainWindow::on_actionHelp_triggered()
{
    helpDialog = new Help;
    helpDialog->exec();
}

void MainWindow::on_actionAbout_triggered()
{
    aboutDialog = new About;
    aboutDialog->exec();
}

void MainWindow::on_actionDonate_triggered()
{
    QDesktopServices::openUrl(QUrl("https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=sascha.makela@gmail.com&lc=LT&item_name=STM%20Software%20-%20Currency%20Converter&currency_code=EUR&bn=PP%2dDonationsBF%3abtn_donateCC_LG%2egif%3aNonHosted"));
}

void MainWindow::on_pushButton_Update_clicked()
{
    ui->lineEdit->clear();
    ui->labelSmall->clear();
    updateCurrency();
}
